'PIO measure pause (pulse low) from GP0
'The program uses a PMW at GP0 output to generate a test frequency, just to test the function

' The PIO program -------------------------------------------------------------------

' configure the PIO 1 sequencer 1 clock and pin registers
p1=0				                  'no IO pins used except PIN in JMP (not part of this register)
e1=Pio(execctrl 0,6,11) 	      'use pin gp0, wrap target = 6, wrap = 11
f1=63e6                       		'63MHz PIO frequency for high resolution

' PIO1 program, measure pause on GP0 (depends on PIN setting EXECCTRL register)
' The pause is 2*TC*count%, where TC = PIO clock frequency
' The state machine code starts at address 6 since adress 0..5 are occupied by the PULSE 
' measurement designed previously, running on state machine 0.
'
'
'line   code    comment
'.wrap target
' 6     A02B    Mov X=-NULL 
' 7     2000    Wait until GP0=0
' 8     00CA    JMP (GP0=1) 10     (jump out of the loop when GP0=1)
' 9     0048    JMP (X<>0) X--, 2  (decrement X and always jump back to 8)
' A     A0C9    MOV ISR = -X       (invert result)
' B     8000    Push noblock       (ISR->FIFO)
'.wrap target

PIO program line 1,6,&hA02B
PIO program line 1,7,&h2000
PIO program line 1,8,&h00CA
PIO program line 1,9,&h0048
PIO program line 1,10,&hA0C9
PIO program line 1,11,&h8000

'PIO1 and setup the machine, and start it from address 6.
PIO init machine 1,1,f1,p1,e1,,6
PIO start 1,1


'The MMBasic program --------------------------------------------------------

'generate 1khz test tone on GP0, 40% (400us) high, 60% (600us) low
SetPin gp0,pwm
PWM 0,1000,40

'The actual measurement math in MMbasic
DIM count%

'read the top of the PIO FIFO one word at at time. Since you start measuring at a 
'random moment, the first reading can be wrong (incomplete cycle).
Do
  PIO READ 1,1,1,count%     'reads the fifo register
  pausewidth = 2 * (1e6/f1) * count% 
  print "the pause width measured is ";pausewidth;" us"
  Pause 100
Loop While Inkey$=""

PIO stop 1,1
End
